{******************************************************************************}
{*                                                                            *}
{* Component: OLE OpenOffice                                  *}
{* Author:                                                       *}
{*                                                                            *}
{* Email:mail@silentium.ru                                                    *}
{* htttp://www.silentium.ru                                                   *}
{* tel: +7-4912-25-17-50                                                      *}
{* Copyright (c) Silentium 2007                                               *}
{*                                                                            *}
{******************************************************************************}
{*                                                                            *}
{*                                                                            *}
{******************************************************************************}

unit SiOpenOffice;

interface

uses
  Classes, ComObj, SysUtils, Dialogs;
// Variants  

const
  //    OpenOffice
  SIOPENOFFICE_CREATEOLEOBJECT = 'com.sun.star.ServiceManager';
  SIOPENOFFICE_CREATEDESKTOP   = 'com.sun.star.frame.Desktop';
  SIOPENOFFICE_PROPERTY_VALUE  = 'com.sun.star.beans.PropertyValue';
  SIOPENOFFICE_DISPATCHHELPER  = 'com.sun.star.frame.DispatchHelper';
  //  
  SIOPENOFFICE_EXCEL97         = 'MS Excel 97';
  SIOPENOFFICE_OOCALC          = 'StarCalc 5.0';
  SIOPENOFFICE_FILE_PREFIX     = 'file:///';
  SIOPENOFFICE_FILTERNAME      = 'FilterName';
  SIOPENOFFICE_BLANK_PARAM     = '_blank';
  SIOPENOFFICE_EXCEL_FACTORY   = 'private:factory/scalc';
  SIOPENOFFICE_INSERTRAWS      = '.uno:InsertRows';
  SIOPENOFFICE_INSERTCOLS      = '.uno:InsertColumns';
  SIOPENOFFICE_GOTOCELL        = '.uno:GoToCell';
  SIOPENOFFICE_TOPOINT         = 'ToPoint';
  SIOPENOFFICE_OVERWRITE       = 'Overwrite';
  SIOPENOFFICE_FILETYPE        = ' MS Excel|*.xls';
  SIOPENOFFICE_FILEEXTENTIOM   = 'xls';
  SIOPENOFFICE_DEFAULTFILENAME = '1';
  // 
  SIOPENOFFICE_MESSAGE_NOSUCHFILE       = '   :';
  SIOPENOFFICE_MESSAGE_OFFICENOTCREATED = 'OpenOffice  ';
  SIOPENOFFICE_MESSAGE_SHEETNOTFOUND    = '   : ';
  SIOPENOFFICE_MESSAGE_CELLNOTFOUND     = '   : ';
  SIOPENOFFICE_MESSAGE_DOCUMENTNOTFOUND = ' ';

type

  TSiOpenOffice = class(TComponent)
  private
    FOffice: Variant;
    FDesctop, FDocument, FSheets: Variant;
    FActiveSheet: Variant;
    FSheetName: string;
    FSheetIndex: Word;
    //      
    FFileName: string;
    FVisible: Boolean;
    //    
    procedure InitSheets;
    procedure SetSheetName(const Value: string);
    procedure SetSheetIndex(const Value: Word);
    // .  
    procedure RunDispatch(ACommand: string; AParams: Variant);
    // .      Unix
    function FileNameToURL(FileName: string): string;
    // .   
    function MakepPropertyValue(APropertyName: string; APropertyValue: variant): Variant;
  public
    constructor Create(AOwner: TComponent);
    //    
    procedure WriteCellText(AValue: string; X, Y: LongInt);
    //    
    procedure WriteCellFormula(AValue: string; X, Y: LongInt);
    //       
    procedure WriteCellTextByName(AName, AValue: string);
    //  
    procedure InsertRows(ACount: LongInt);
    //  
    procedure InsertCols(ACount: LongInt);
    //      
    procedure GoToCell(X, Y: LongInt);
    // 
    function Connect: Boolean;
    // 
    function Disconnect: Boolean;
    //  
    function CreateFile: Boolean;
    //  
    function OpenFile(AFileName: string): Boolean;
    //  
    function SaveFile: Boolean;
    //     
    function SaveAsFile(AFileName: string): Boolean;
    //    
    property SheetName: string read FSheetName write SetSheetName;
    //    
    property SheetIndex: Word read FSheetIndex write SetSheetIndex;
    //   
    property Visible: Boolean read FVisible write FVisible;
  published
    {}
  end;

  //     
  function SiOpenOfficeLetter(Value: LongInt): string;
  //      
  function FileType(AFileName: string): string;

implementation


function TSiOpenOffice.Connect: Boolean;
begin
  if VarIsEmpty(FOffice) then
   FOffice := CreateOLEObject(SIOPENOFFICE_CREATEOLEOBJECT);
  Result := true;
end;

function TSiOpenOffice.Disconnect: Boolean;
begin
  FOffice := unAssigned;
  Result := true;
end;

function TSiOpenOffice.CreateFile: Boolean;
begin
  FDesctop := FOffice.CreateInstance(SIOPENOFFICE_CREATEDESKTOP);
  FDocument := FDesctop.LoadComponentFromURL(
               SIOPENOFFICE_EXCEL_FACTORY, SIOPENOFFICE_BLANK_PARAM, 0,
               VarArrayCreate([0, -1], varVariant));
  FFileName := '';
  //   
  InitSheets;
  Result := true;
end;

function TSiOpenOffice.OpenFile(AFileName: string): Boolean;
var
  xVariantArray: Variant;
  xFileName: string;
  xFileType: string;
begin
  Result := false;
  //    
  if VarIsEmpty(FOffice) then
  begin
    ShowMessage(SIOPENOFFICE_MESSAGE_OFFICENOTCREATED);
    Exit;
  end;
  if FileExists(AFileName) then
  begin
    //   
    xFileType := FileType(AFileName);
    // SIOPENOFFICE_EXCEL97
    xFileName := FileNameToURL(AFileName);
    FDesctop := FOffice.CreateInstance(SIOPENOFFICE_CREATEDESKTOP);
    xVariantArray := VarArrayCreate([0, 0], varVariant);
    xVariantArray[0] := MakepPropertyValue(SIOPENOFFICE_FILTERNAME , xFileType);
    FDocument := FDesctop.LoadComponentFromURL(xFileName, SIOPENOFFICE_BLANK_PARAM, 0, xVariantArray);
    FFileName := AFileName;
    //   
    InitSheets;
    Result := true;
  end
  else
    ShowMessage(SIOPENOFFICE_MESSAGE_NOSUCHFILE + ' ' + AFileName);
end;

function FileType(AFileName: string): string;
var
  xLen: LongInt;
  xFileName, xExtention: string;
begin
  xFileName := AFileName;
  xLen := Length(xFileName);
  xExtention := copy(xFileName, xLen - 3, 3);
  if (xExtention = 'ods') or (xExtention = 'odt') then
    Result := SIOPENOFFICE_OOCALC;
  if (xExtention = 'xls') or (xExtention = 'xlt') then
    Result := SIOPENOFFICE_EXCEL97;
end;

function TSiOpenOffice.SaveFile: Boolean;
var
  xVariantArray: Variant;
  xFileName: string;
  xDialog: TSaveDialog;
  xFileType: string;
begin
  Result := false;
  if VarIsEmpty(FDocument) then
  begin
    ShowMessage(SIOPENOFFICE_MESSAGE_DOCUMENTNOTFOUND);
    Exit;
  end;
  //      ,   ...
  if FFileName = '' then
  begin
    xDialog := TSaveDialog.Create(Self);
    xDialog.Filter := SIOPENOFFICE_FILETYPE;
    xDialog.DefaultExt := SIOPENOFFICE_FILEEXTENTIOM;
    xDialog.Options := xDialog.Options + [ofOverwritePrompt];
    xDialog.FileName := SIOPENOFFICE_DEFAULTFILENAME;
    if xDialog.Execute then
      FFileName := xDialog.FileName;
    xDialog.Free;
  end;
  //    
  if FFileName <> '' then
  begin
    //   
    xFileType := FileType(FFileName);
    xFileName := FileNameToURL(FFileName);
    xVariantArray := VarArrayCreate([0, 0], varVariant);
    xVariantArray[0] := MakepPropertyValue(SIOPENOFFICE_FILTERNAME , xFileType);
    FDocument.StoreToURL(xFileName, xVariantArray);
  end;
end;

function TSiOpenOffice.SaveAsFile(AFileName: string): Boolean;
var
  xVariantArray: Variant;
  xFileName: string;
  xFileType: string;
begin
  Result := false;
  if VarIsEmpty(FDocument) then
  begin
    ShowMessage(SIOPENOFFICE_MESSAGE_DOCUMENTNOTFOUND);
    Exit;
  end;
  xFileType := FileType(AFileName);
  xFileName := FileNameToURL(AFileName);
  xVariantArray := VarArrayCreate([0, 1], varVariant);
  xVariantArray[0] := MakepPropertyValue(SIOPENOFFICE_FILTERNAME , xFileType);
  xVariantArray[1] := MakepPropertyValue(SIOPENOFFICE_OVERWRITE , True);
  FDocument.StoreToURL(xFileName, xVariantArray);
  FFileName := AFileName;
  Result := true;
end;

function TSiOpenOffice.FileNameToURL(FileName: string): string;
begin
  Result:= '';
  if LowerCase(copy(FileName,1,8))<>SIOPENOFFICE_FILE_PREFIX then
    Result:= SIOPENOFFICE_FILE_PREFIX;
  Result:= Result + StringReplace(LowerCase(FileName), '\', '/', [rfReplaceAll, rfIgnoreCase]);
end;

procedure TSiOpenOffice.InitSheets;
begin
  FSheets := FDocument.GetSheets;
  FActiveSheet := FSheets.GetByIndex(0);
end;

procedure TSiOpenOffice.InsertRows(ACount: Integer);
var
  i: LongInt;
begin
  for i := 0 to ACount - 1 do
    RunDispatch(SIOPENOFFICE_INSERTRAWS, unAssigned);
end;

procedure TSiOpenOffice.InsertCols(ACount: Integer);
var
  i: LongInt;
begin
  for i := 0 to ACount - 1 do
    RunDispatch(SIOPENOFFICE_INSERTCOLS, unAssigned);
end;

function TSiOpenOffice.MakepPropertyValue(APropertyName: string; APropertyValue: variant): Variant;
var
  xStructure: variant;
begin
  xStructure := FOffice.Bridge_GetStruct(SIOPENOFFICE_PROPERTY_VALUE);
  xStructure.Name := APropertyName;
  xStructure.Value := APropertyValue;
  Result := xStructure;
end;

procedure TSiOpenOffice.RunDispatch(ACommand: string; AParams: Variant);
var
  xDispatcher, xFrame: variant;
begin
  if (VarIsEmpty(AParams) or VarIsNull(AParams)) then
      AParams:= VarArrayCreate([0, -1], varVariant);
  xFrame:= FDocument.getCurrentController.getFrame;
  xDispatcher:= FOffice.createInstance(SIOPENOFFICE_DISPATCHHELPER);
  xDispatcher.executeDispatch(xFrame, ACommand, '', 0, AParams);
end;

procedure TSiOpenOffice.SetSheetIndex(const Value: Word);
begin
  FActiveSheet := FSheets.getByIndex(Value);
  FSheetIndex := Value;
end;

procedure TSiOpenOffice.SetSheetName(const Value: string);
begin
  try
    FActiveSheet := FSheets.getByName(Value);
    FSheetName := Value;
  except
    ShowMessage(SIOPENOFFICE_MESSAGE_SHEETNOTFOUND + Value);
  end;
end;

procedure TSiOpenOffice.WriteCellText(AValue: string; X, Y: Integer);
var
  xCell: Variant;
begin
  // 
  xCell := FActiveSheet.GetCellByPOsition(X, Y);
  //     
  xCell.SetString(AValue);
end;

procedure TSiOpenOffice.WriteCellFormula(AValue: string; X, Y: Integer);
var
  xCell: Variant;
begin
 // 
  xCell := FActiveSheet.GetCellByPOsition(X, Y);
  //     
  xCell.SetFormula(AValue);
end;

procedure TSiOpenOffice.WriteCellTextByName(AName, AValue: string);
var
  xCell: Variant;
begin
  //         AName  Insert/Names/Define
  try
    xCell := FActiveSheet.GetCellRangeByName(AName);
    xCell.SetString(AValue);
  except
    ShowMessage(SIOPENOFFICE_MESSAGE_CELLNOTFOUND + AName);
  end;
end;

procedure TSiOpenOffice.GoToCell(X, Y: Integer);
var
  xParams: Variant;
  xCellString: string;
begin
  xCellString := '$' + SiOpenOfficeLetter(X) + '$' + IntToStr(Y);   // '$A$8'
  xParams:= VarArrayCreate([0, 0], varVariant);
  xParams[0]:= MakepPropertyValue(SIOPENOFFICE_TOPOINT, xCellString);
  RunDispatch(SIOPENOFFICE_GOTOCELL, xParams);
end;

function SiOpenOfficeLetter(Value: LongInt): string;
begin
  Result := '';
  case Value of
    1:  Result := 'A';
    2:  Result := 'B';
    3:  Result := 'C';
    4:  Result := 'D';
    5:  Result := 'E';
    6:  Result := 'F';
    7:  Result := 'G';
    8:  Result := 'H';
    9:  Result := 'I';
    10: Result := 'J';
    11: Result := 'K';
    12: Result := 'L';
    13: Result := 'M';
    14: Result := 'N';
    15: Result := 'O';
    16: Result := 'P';
    17: Result := 'Q';
    18: Result := 'R';
    19: Result := 'S';
    20: Result := 'T';
    21: Result := 'U';
    22: Result := 'V';
    23: Result := 'W';
    24: Result := 'X';
    25: Result := 'Y';
    26: Result := 'Z';
    27: Result := 'AA';
    28: Result := 'AB';
    29: Result := 'AC';
    30: Result := 'AD';
    31: Result := 'AE';
    32: Result := 'AF';
    33: Result := 'AG';
    34: Result := 'AH';
    35: Result := 'AI';
    36: Result := 'AJ';
    37: Result := 'AK';
    38: Result := 'AL';
    39: Result := 'AM';
    40: Result := 'AN';
    41: Result := 'AO';
    42: Result := 'AP';
    43: Result := 'AQ';
    44: Result := 'AR';
    45: Result := 'AS';
    46: Result := 'AT';
    47: Result := 'AU';
    48: Result := 'AV';
    49: Result := 'AW';
    50: Result := 'AX';
    51: Result := 'AY';
    52: Result := 'AZ';
    53: Result := 'BA';
    54: Result := 'BB';
    55: Result := 'BC';
    56: Result := 'BD';
    57: Result := 'BE';
    58: Result := 'BF';
    59: Result := 'BG';
    60: Result := 'BH';
    61: Result := 'BI';
    62: Result := 'BJ';
    63: Result := 'BK';
    64: Result := 'BL';
    65: Result := 'BM';
    66: Result := 'BN';
    67: Result := 'BO';
    68: Result := 'BP';
    69: Result := 'BQ';
    70: Result := 'BR';
    71: Result := 'BS';
    72: Result := 'BT';
    73: Result := 'BU';
    74: Result := 'BV';
    75: Result := 'BW';
    76: Result := 'BX';
    77: Result := 'BY';
    78: Result := 'BZ';
  end;
end;

constructor TSiOpenOffice.Create(AOwner: TComponent);
begin
  inherited;
  FVisible := true;
end;

end.
